/*
------------------------------------------------------------
   Kobo Deluxe - An enhanced SDL port of XKobo
------------------------------------------------------------
 * Copyright (C) 2001, David Olofson
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "glSDL.h"
#include <math.h>

#include "config.h"
#include "states.h"
#include "kobo.h"
#include "screen.h"
#include "manage.h"
#include "sounds.h"
#include "options.h"

gamestatemanager_t gsm;

int mouse_x = 0;
int mouse_y = 0;

int run_demo = 0;


kobo_basestate_t::kobo_basestate_t()
{
	name = "<unnamed>";
}


void kobo_basestate_t::frame()
{
}


void kobo_basestate_t::pre_render(window_t *win)
{
	screen.render_background(win);
}


void kobo_basestate_t::post_render(window_t *win)
{
	DBG(
		win->font(B_NORMAL_FONT);
		win->string(30, 5, name);
	)
}


/*----------------------------------------------------------
	st_introbase
----------------------------------------------------------*/

st_introbase_t::st_introbase_t()
{
	name = "intro";
	inext = NULL;
	duration = 0;
	timer = 0;
}


void st_introbase_t::enter()
{
	if(!run_demo)
	{
		manage.init();
		manage.init_resources_title();
		run_demo = 1;
	}
	start_time = (int)SDL_GetTicks();
	timer = 0;
}


void st_introbase_t::reenter()
{
	if(!run_demo)
	{
		manage.init_resources_title();
		run_demo = 1;
	}

	if(this != &st_intro_1)
		gsm.change(&st_intro_1);
}


void st_introbase_t::press(int button)
{
	switch (button)
	{
	  case BTN_EXIT:
	  case BTN_CLOSE:
		gsm.push(&st_ask_exit);
		break;
	  case BTN_START:
	  case BTN_FIRE:
	  case BTN_SELECT:
		gsm.push(&st_main_menu);
		break;
	  case BTN_UP:
	  case BTN_INC:
		manage.select_next();
		break;
	  case BTN_DOWN:
	  case BTN_DEC:
		manage.select_prev();
		break;
	  case BTN_REGEN:
		break;
	  case BTN_LEFT:
	  case BTN_RIGHT:
		if(inext)
			gsm.change(inext);
		break;
	  default:
		break;
	}
}


void st_introbase_t::frame()
{
	timer = (int)SDL_GetTicks() - start_time;
	manage.run_demo();
}


void st_introbase_t::post_render(window_t *win)
{
	kobo_basestate_t::post_render(win);
	screen.scroller();
//	screen.usage();

	//This should be in frame(), but that makes
	//change() freak out sometimes, it seems...
	if((timer > duration) && inext)
		gsm.change(inext);
}




/*----------------------------------------------------------
	st_intro_1
----------------------------------------------------------*/

st_intro_1_t::st_intro_1_t()
{
	name = "intro_1 (title)";
}

void st_intro_1_t::enter()
{
	st_introbase_t::enter();
	if(!duration)
		duration = INTRO_TITLE_TIME;
	if(!inext)
		inext = &st_intro_2;
}

void st_intro_1_t::post_render(window_t *win)
{
	if(exit_game)
		return;
	st_introbase_t::post_render(win);
	if(timer < duration - INTRO_BLANK_TIME)
		screen.title();
}

st_intro_1_t st_intro_1;




/*----------------------------------------------------------
	st_intro_2
----------------------------------------------------------*/

st_intro_2_t::st_intro_2_t()
{
	name = "intro_2 (highscores)";
}

void st_intro_2_t::enter()
{
	st_introbase_t::enter();
	duration = INTRO_HIGHSCORE_TIME;
	inext = &st_intro_1;
	st_intro_1.inext = &st_intro_3;
	st_intro_1.duration = INTRO_TITLE2_TIME;
}

void st_intro_2_t::post_render(window_t *win)
{
	if(exit_game)
		return;
	st_introbase_t::post_render(win);
	if(timer < duration - INTRO_BLANK_TIME)
		screen.highscores();
}

st_intro_2_t st_intro_2;




/*----------------------------------------------------------
	st_intro_3
----------------------------------------------------------*/

st_intro_3_t::st_intro_3_t()
{
	name = "intro_3 (credits)";
}

void st_intro_3_t::enter()
{
	st_introbase_t::enter();
	duration = INTRO_CREDITS_TIME;
	inext = &st_intro_1;
	st_intro_1.inext = &st_intro_2;
	st_intro_1.duration = INTRO_TITLE_TIME;
}

void st_intro_3_t::post_render(window_t *win)
{
	if(exit_game)
		return;
	st_introbase_t::post_render(win);
	if(timer < duration - INTRO_BLANK_TIME)
		screen.credits();
}

st_intro_3_t st_intro_3;




/*----------------------------------------------------------
	st_ask_exit
----------------------------------------------------------*/

st_ask_exit_t::st_ask_exit_t()
{
	name = "ask_exit";
}


void st_ask_exit_t::enter()
{
	play_sound(SOUND_EXPL);
}


void st_ask_exit_t::press(int button)
{
	switch (button)
	{
	  case BTN_EXIT:
	  case BTN_NO:
		play_sound(SOUND_SHOT);
		pop();
		break;
	  case BTN_CLOSE:
	  case BTN_YES:
		play_sound(SOUND_EXPL);
		exit_game = 1;
		pop();
		break;
	}
}


void st_ask_exit_t::frame()
{
	if(run_demo)
		manage.run_demo();
}


void st_ask_exit_t::post_render(window_t *win)
{
	kobo_basestate_t::post_render(win);

	float ft = SDL_GetTicks() * 0.001;
	win->font(B_BIG_FONT);
	int y = PIXEL2CS(100) + (int)floor(PIXEL2CS(15)*sin(ft * 6));
	win->center_fxp(y, "Quit Kobo Deluxe?\n");
}

st_ask_exit_t st_ask_exit;




/*----------------------------------------------------------
	st_game
----------------------------------------------------------*/

st_game_t::st_game_t()
{
	name = "game";
}


void st_game_t::enter()
{
	run_demo = 0;
	manage.game_start();
}


void st_game_t::leave()
{
	manage.game_stop();
}


void st_game_t::yield()
{
	manage.pause_music(1);
}


void st_game_t::reenter()
{
	manage.pause_music(0);
}


void st_game_t::press(int button)
{
	switch (button)
	{
	  case BTN_EXIT:
		gsm.push(&st_ask_abort_game);
		break;
	  case BTN_CLOSE:
		gsm.push(&st_ask_exit);
		break;
	  case BTN_SELECT:
	  case BTN_START:
		gsm.push(&st_pause_game);
		break;
	}
}


void st_game_t::frame()
{
	manage.run_game();
	if(manage.game_over())
		gsm.push(&st_game_over);
	else if(manage.get_ready())
	{
		st_get_ready.update_kludge = 1;
		gsm.push(&st_get_ready);
	}
	else if(exit_game || manage.game_stopped())
		pop();
}


void st_game_t::post_render(window_t *win)
{
	kobo_basestate_t::post_render(win);
}

st_game_t st_game;



/*----------------------------------------------------------
	st_ask_abort_game
----------------------------------------------------------*/

st_ask_abort_game_t::st_ask_abort_game_t()
{
	name = "ask_abort_game";
}


void st_ask_abort_game_t::enter()
{
	play_sound(SOUND_EXPL);
}


void st_ask_abort_game_t::press(int button)
{
	switch (button)
	{
	  case BTN_EXIT:
	  case BTN_NO:
		play_sound(SOUND_SHOT);
		gsm.change(&st_get_ready);
//		pop();
		break;
	  case BTN_CLOSE:
	  case BTN_YES:
		play_sound(SOUND_EXPL);
		manage.game_stop();
		pop();
		break;
	}
}


void st_ask_abort_game_t::frame()
{
}


void st_ask_abort_game_t::post_render(window_t *win)
{
	kobo_basestate_t::post_render(win);

	float ft = SDL_GetTicks() * 0.001;
	win->font(B_BIG_FONT);
	int y = PIXEL2CS(80) + (int)floor(PIXEL2CS(15)*sin(ft * 6));
	win->center_fxp(y, "Abort Game?\n");
}

st_ask_abort_game_t st_ask_abort_game;



/*----------------------------------------------------------
	st_pause_game
----------------------------------------------------------*/

st_pause_game_t::st_pause_game_t()
{
	name = "pause_game";
}


void st_pause_game_t::enter()
{
	play_sound(SOUND_EXPL);
}


void st_pause_game_t::press(int button)
{
	switch (button)
	{
	  case BTN_EXIT:
		gsm.change(&st_ask_abort_game);
		break;
	  case BTN_SELECT:
	  case BTN_START:
	  case BTN_YES:
	  case BTN_NO:
		gsm.change(&st_get_ready);
		break;
	}
}


void st_pause_game_t::frame()
{
}


void st_pause_game_t::post_render(window_t *win)
{
	kobo_basestate_t::post_render(win);

	float ft = SDL_GetTicks() * 0.001;
	win->font(B_BIG_FONT);
	int y = PIXEL2CS(75) + (int)floor(PIXEL2CS(15)*sin(ft * 6));
	win->center_fxp(y, "PAUSED");
}

st_pause_game_t st_pause_game;



/*----------------------------------------------------------
	st_get_ready
----------------------------------------------------------*/

st_get_ready_t::st_get_ready_t()
{
	name = "get_ready";
}


void st_get_ready_t::enter()
{
	play_sound(SOUND_EXPL2);
	start_time = (int)SDL_GetTicks();
	frame_time = 0;
	countdown = 9;
//	update_kludge = 1;
}


void st_get_ready_t::reenter()
{
	//Countdown should pause while in some sub state.
//Note: Now, this code won't be called, as all "pause style"
//	states now *change()* to "get ready" rather than
//	right into the game. 
	start_time = (int)SDL_GetTicks() - frame_time;
	frame_time = 0;
}


void st_get_ready_t::press(int button)
{
	if(frame_time < 500)
		return;

	switch (button)
	{
	  case BTN_EXIT:
	  case BTN_NO:
		gsm.change(&st_ask_abort_game);
		break;
	  case BTN_LEFT:
	  case BTN_RIGHT:
	  case BTN_UP:
	  case BTN_DOWN:
	  case BTN_UL:
	  case BTN_UR:
	  case BTN_DL:
	  case BTN_DR:
	  case BTN_FIRE:
	  case BTN_YES:
		play_sound(SOUND_EXPL);
		pop();
		break;
	  case BTN_SELECT:
	  case BTN_START:
		gsm.change(&st_pause_game);
		break;
	}
}


void st_get_ready_t::frame()
{
	if(exit_game || manage.game_stopped())
	{
		pop();
		return;
	}

	if(update_kludge)
	{
		manage.run_game();
		update_kludge = 0;
	}

	frame_time = (int)SDL_GetTicks() - start_time;

	int prevcount = countdown;
	countdown = 9 - frame_time/1000;
	if(prevcount != countdown)
		play_sound(SOUND_METALLIC);

	if(countdown < 1)
	{
//		play_sound(SOUND_EXPL);
//		pop();
		gsm.change(&st_pause_game);
	}
}


void st_get_ready_t::post_render(window_t *win)
{
	kobo_basestate_t::post_render(win);

//	if(frame_time < 300)
//		return;

	float ft = SDL_GetTicks() * 0.001;
	char counter[2] = "0";
	win->font(B_BIG_FONT);
	int y = PIXEL2CS(75) + (int)floor(PIXEL2CS(15)*sin(ft * 6));
	win->center_fxp(y, "GET READY!");
	counter[0] = countdown + '0';
	win->font(B_COUNTER_FONT);
	win->center_fxp(y + PIXEL2CS(65), counter);
}

st_get_ready_t st_get_ready;


/*----------------------------------------------------------
	st_game_over
----------------------------------------------------------*/

st_game_over_t::st_game_over_t()
{
	name = "game_over";
}


void st_game_over_t::enter()
{
	start_time = (int)SDL_GetTicks();
}


void st_game_over_t::press(int button)
{
	if(frame_time < 500)
		return;

	switch (button)
	{
	  case BTN_EXIT:
	  case BTN_NO:
	  case BTN_LEFT:
	  case BTN_RIGHT:
	  case BTN_UP:
	  case BTN_DOWN:
	  case BTN_UL:
	  case BTN_UR:
	  case BTN_DL:
	  case BTN_DR:
	  case BTN_FIRE:
	  case BTN_START:
	  case BTN_SELECT:
	  case BTN_YES:
//		play_sound(SOUND_EXPL);
		pop();
		break;
	}
}


void st_game_over_t::frame()
{
	frame_time = (int)SDL_GetTicks() - start_time;

	if(frame_time > 5000)
		pop();
}


void st_game_over_t::post_render(window_t *win)
{
	kobo_basestate_t::post_render(win);

//	if(frame_time < 300)
//		return;

	float ft = SDL_GetTicks() * 0.001;
	win->font(B_BIG_FONT);
	int y = PIXEL2CS(100) + (int)floor(PIXEL2CS(15)*sin(ft * 6));
	win->center_fxp(y, "GAME OVER");
}

st_game_over_t st_game_over;



/*----------------------------------------------------------
	st_main_menu
----------------------------------------------------------*/
void main_menu_t::change(int delta)
{
	kobo_form_t::change(delta);

	if(!selected())
		return;

	selection = selected()->tag;
}


void main_menu_t::build()
{
	form_big();
	form_space();
	form_space();
	form_space();
		form_button("Start Game!", 1);
		form_button("Options", 2);
	form_space();
		form_button("Return to Intro", 0);
	form_space();
		form_button("Quit Kobo Deluxe", 100);
}


void st_main_menu_t::press(int button)
{
	menu.selection = -1;

	switch (button)
	{
	  case BTN_EXIT:
	  case BTN_CLOSE:
		menu.selection = 0;
		break;
	  case BTN_INC:
	  case BTN_DEC:
	  case BTN_FIRE:
	  case BTN_START:
	  case BTN_SELECT:
		menu.change(0);
		break;
	  case BTN_UP:
		menu.prev();
		break;
	  case BTN_DOWN:
		menu.next();
		break;
	}

	switch(menu.selection)
	{
	  case 0:
		play_sound(SOUND_SHOT);
		menu.close();
		pop();
		break;
	  case 1:
		play_sound(SOUND_EXPL);
		gsm.change(&st_game);
		break;
	  case 2:
		play_sound(SOUND_EXPL);
		gsm.push(&st_options_main);
		break;
	  case 100:
		play_sound(SOUND_EXPL);
		gsm.change(&st_ask_exit);
		break;
	}
}


void main_menu_t::open()
{
	init(&gengine);
	place(wchip.x(), wchip.y(), wchip.width(), wchip.height());
	font(B_NORMAL_FONT);
	foreground(wbase.alloc_color(65535, 65535, 65535));
	background(wbase.alloc_color(0, 0, 0));
	build_all();
}


void main_menu_t::close()
{
	clean();
}


st_main_menu_t::st_main_menu_t()
{
	name = "main_menu";
}


void st_main_menu_t::enter()
{
	menu.open();
	run_demo = 1;
	play_sound(SOUND_EXPL);
}


// Because we may get back here after changing the configuration!
void st_main_menu_t::reenter()
{
	if(main_options.status() & OS_RESTART_VIDEO)
		pop();
}


void st_main_menu_t::leave()
{
	menu.close();
}


void st_main_menu_t::frame()
{
	manage.run_demo();
}


void st_main_menu_t::post_render(window_t *win)
{
	kobo_basestate_t::post_render(win);
	menu.render();
}

st_main_menu_t st_main_menu;




/*----------------------------------------------------------
	st_options_main
----------------------------------------------------------*/

st_options_main_t::st_options_main_t()
{
	name = "options_main";
}


void st_options_main_t::enter()
{
	main_options.init(&gengine);
	main_options.place(wchip.x(), wchip.y(), wchip.width(), wchip.height());
	main_options.font(B_NORMAL_FONT);
	main_options.foreground(wbase.alloc_color(65535, 65535, 65535));
	main_options.background(wbase.alloc_color(0, 0, 0));
	main_options.open(&prefs);
	run_demo = 1;
	play_sound(SOUND_EXPL);
}


void st_options_main_t::leave()
{
	main_options.close();
}


void st_options_main_t::press(int button)
{
	switch (button)
	{
	  case BTN_EXIT:
	  case BTN_CLOSE:
		play_sound(SOUND_SHOT);
		main_options.undo();
		main_options.close();
		pop();
		break;
	  case BTN_INC:
	  case BTN_RIGHT:
		main_options.change(1);
		break;
	  case BTN_DEC:
	  case BTN_LEFT:
		main_options.change(-1);
		break;
	  case BTN_FIRE:
	  case BTN_START:
	  case BTN_SELECT:
		main_options.change(0);
		if(main_options.status() & OS_CLOSE)
		{
			if(main_options.status() & (OS_RESTART | OS_RELOAD))
			{
				exit_game = 0;
				manage.abort();
			}
			main_options.close();
			pop();
		}
		break;
	  case BTN_UP:
		main_options.prev();
		break;
	  case BTN_DOWN:
		main_options.next();
		break;
	}

	/* 
	 * Handle changes that require only an update...
	 */
	if(main_options.status() & OS_UPDATE_AUDIO)
	{
		audio_master_volume(prefs.volume);
		audio_master_reverb(prefs.reverb);
		audio_set_limiter((float)prefs.threshold / 100.0,
				(float)prefs.release / 100.0);
	}
	if(main_options.status() & OS_UPDATE_ENGINE)
	{
		gengine.period(prefs.wait_msec);
		gengine.interpolation(prefs.filter);
	}
	main_options.clearstatus(OS_UPDATE);
}


void st_options_main_t::frame()
{
	manage.run_demo();
}


void st_options_main_t::post_render(window_t *win)
{
	kobo_basestate_t::post_render(win);
	main_options.render();
}

st_options_main_t st_options_main;
