/*
------------------------------------------------------------
   Kobo Deluxe - An enhanced SDL port of XKobo
------------------------------------------------------------
 * Copyright (C) 2001, David Olofson
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef	_KOBO_STATES_H_
#define	_KOBO_STATES_H_

#include "gamestate.h"
#include "gamectl.h"
#include "form.h"

class kobo_basestate_t : public gamestate_t
{
  public:
	kobo_basestate_t();

	void frame();
	void pre_render(window_t *win);
	void post_render(window_t *win);
};


extern int mouse_x;
extern int mouse_y;

extern gamestatemanager_t gsm;


/*----------------------------------------------------------
	Intro
----------------------------------------------------------*/

class st_introbase_t : public kobo_basestate_t
{
  protected:
	int		timer, start_time;
  public:
	int		duration;
	st_introbase_t	*inext;
	st_introbase_t();
	void enter();
	void reenter();
	void press(int button);
	void frame();
	void post_render(window_t *win);
};


class st_intro_1_t : public st_introbase_t
{
  public:
	st_intro_1_t();
	void enter();
	void post_render(window_t *win);
};
extern st_intro_1_t st_intro_1;


class st_intro_2_t : public st_introbase_t
{
  public:
	st_intro_2_t();
	void enter();
	void post_render(window_t *win);
};
extern st_intro_2_t st_intro_2;


class st_intro_3_t : public st_introbase_t
{
  public:
	st_intro_3_t();
	void enter();
	void post_render(window_t *win);
};
extern st_intro_3_t st_intro_3;


class st_ask_exit_t : public kobo_basestate_t
{
  public:
	st_ask_exit_t();
	void enter();
	void press(int button);
	void frame();
	void post_render(window_t *win);
};
extern st_ask_exit_t st_ask_exit;


/*----------------------------------------------------------
	In-game
----------------------------------------------------------*/

class st_game_t : public kobo_basestate_t
{
  public:
	st_game_t();
	void enter();
	void leave();
	void yield();
	void reenter();
	void press(int button);
	void frame();
	void post_render(window_t *win);
};
extern st_game_t st_game;


class st_ask_abort_game_t : public kobo_basestate_t
{
  public:
	st_ask_abort_game_t();
	void enter();
	void press(int button);
	void frame();
	void post_render(window_t *win);
};
extern st_ask_abort_game_t st_ask_abort_game;


class st_pause_game_t : public kobo_basestate_t
{
  public:
	st_pause_game_t();
	void enter();
	void press(int button);
	void frame();
	void post_render(window_t *win);
};
extern st_pause_game_t st_pause_game;


class st_get_ready_t : public kobo_basestate_t
{
	int	start_time, frame_time;
	int	countdown;
  public:
	int	update_kludge;
	st_get_ready_t();
	void enter();
	void reenter();
	void press(int button);
	void frame();
	void post_render(window_t *win);
};
extern st_get_ready_t st_get_ready;


class st_game_over_t : public kobo_basestate_t
{
	int	start_time, frame_time;
  public:
	st_game_over_t();
	void enter();
	void press(int button);
	void frame();
	void post_render(window_t *win);
};
extern st_game_over_t st_game_over;


/*----------------------------------------------------------
	Main Menu
----------------------------------------------------------*/
class main_menu_t : public kobo_form_t
{
  public:
	int	selection;
	virtual void change(int delta);
	virtual void build();
	void open();
	void close();
};

class st_main_menu_t : public kobo_basestate_t
{
	main_menu_t	menu;
  public:
	st_main_menu_t();
	void enter();
	void reenter();
	void leave();
	void press(int button);
	void frame();
	void post_render(window_t *win);
};
extern st_main_menu_t st_main_menu;


/*----------------------------------------------------------
	Options
----------------------------------------------------------*/

class st_options_main_t : public kobo_basestate_t
{
  public:
	st_options_main_t();
	void enter();
	void leave();
	void press(int button);
	void frame();
	void post_render(window_t *win);
};
extern st_options_main_t st_options_main;

#endif	//_KOBO_STATES_H_
